from lxml import etree
from PIL import Image
import uuid
import base64
from io import BytesIO

from ..utils import process_transform
from .default import DefaultTagProcessor
from .common import NAMESPACEMAP


processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('BitmapImage')
class BitmapImageTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        params.update(process_transform(**target.attrs))
        if 'source' in target.attrs:
            src = target.attrs['source']
            if src.startswith('@Embed'):
                src = src[src.find("(") + 2:src.find(")") - 1]
                params['width'], params['height'] = map(lambda s: "%s" % s, Image.open(src).size)
                # *_, ext = src.split('.')
                # buffer = BytesIO()
                # Image.open(src).save(buffer, ext)
                # src = b"data:image/" + bytes(ext.lower(), encoding='utf8') + b";base64," + base64.b64encode(buffer.getvalue())
            params['{%s}href' % NAMESPACEMAP['xlink']] = src
        if 'width' in target.attrs:
            params['width'] = target.attrs['width']
        if 'height' in target.attrs:
            params['height'] = target.attrs['height']
        target.element = etree.SubElement(
            target.element,
            "image",
            **params
        )



from lxml import etree
import logging

from ..utils import process_transform, populate_transform
from .default import DefaultTagProcessor


logger = logging.getLogger(__name__)


NAMESPACEMAP = {
    None: "http://www.w3.org/2000/svg",
    "xlink" : 'http://www.w3.org/1999/xlink'
}

processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('Graphic')
class GraphicTagProcessor(DefaultTagProcessor):

    def on_start(self, target):
        attrs = target.attrs
        target.element = etree.Element(
            "svg",
            nsmap=NAMESPACEMAP,
            version="1.1",
            width=attrs.get("viewWidth"),
            height=attrs.get("viewHeight"),
            viewBox="0 0 %s %s" % (attrs.get("viewWidth"), attrs.get("viewHeight")),
            # viewport="0 0 %s %s" % (attrs.get("viewWidth"), attrs.get("viewHeight")),
            preserveAspectRatio="none",
            **{"enable-background": "new 0 0 %s %s" % (attrs.get("viewWidth"), attrs.get("viewHeight"))}
        )


@tag_processor('Library')
class LibraryTagProcessor(DefaultTagProcessor):

    def on_start(self, target):
        root = target.root
        target.element = etree.SubElement(root, "defs")


@tag_processor('Group')
class GroupTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        params.update(process_transform(**target.attrs))
        if 'alpha' in target.attrs:
            params['opacity'] = target.attrs['alpha']
        target.element = etree.SubElement(
            target.element,
            "g",
            **params
        )

    def on_end(self, target):
        if 'transform' in target.element.attrib:
            populate_transform(target.element)
        element = target.element
        self.process_children(element)
        parent = element.getparent()
        target.element = parent

    def process_children(self, element):
        logger.debug("Process chidlren %s" % element.tag)
        clippathid = None
        maskedElements = []
        for e in element:
            logger.debug("chidlren %s" % e.tag)
            if clippathid:
                if e.tag == "g":
                    g = e
                    g.attrib['clip-path'] = 'url(#%s)' % clippathid
                else:
                    g = etree.Element(
                        "g",
                        **{'clip-path': 'url(#%s)' % clippathid}
                    )
                    g.append(e)
                maskedElements.append(g)
            if e.tag == "clipPath":
                clippathid = e.attrib['id']
        logger.debug(maskedElements)
        for e in maskedElements:
            element.append(e)



from collections import defaultdict
import logging
from .default import DefaultTagProcessor
from .common import processors as common_processors
from .text import processors as text_processors
from .geometry import processors as geom_processors
from .img import processors as img_processors
from .fill import processors as fill_processors
from .stroke import processors as stroke_processors
from .mask import processors as mask_processors
from .filters import processors as filters_processors


logger = logging.getLogger(__name__)

all_processors = defaultdict(DefaultTagProcessor)
all_processors.update(common_processors)
all_processors.update(text_processors)
all_processors.update(geom_processors)
all_processors.update(img_processors)
all_processors.update(fill_processors)
all_processors.update(stroke_processors)
all_processors.update(mask_processors)
all_processors.update(filters_processors)


def tag_processor(tagname):
    def register_tag_processor(cls):
        all_processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('fill')
@tag_processor('stroke')
@tag_processor('content')
@tag_processor('filters')
@tag_processor('Private')
@tag_processor('FontMap')
@tag_processor('Map')
class IgnoreTagProcessor(object):
    def on_start(self, target):
        logger.debug("IGNORING %s %s" % (target.tagname, target.attrs))

    def on_end(self, target):
        pass

    def on_data(self, target):
        pass



from lxml import etree
import uuid

from ..utils import process_transform
from .default import DefaultTagProcessor


processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


def remove_group_elements(element, tag):
    for e in element:
        if tag == e.tag:
            for ch in e:
                remove_group_elements(ch, tag)
                element.append(ch)
            element.remove(e)


@tag_processor('mask')
class MaskTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        params['id'] = str(uuid.uuid4())
        target.element = etree.SubElement(
            target.element,
            "clipPath",
            # maskUnits="userSpaceOnUse",
            **params
        )

    def on_end(self, target):
        remove_group_elements(target.element, "g")
        super(MaskTagProcessor, self).on_end(target)



import logging


logger = logging.getLogger(__name__)


def process_transform(**attrs):
    transform = []

    if 'x' in attrs or 'y' in attrs:
        x = float(attrs.get('x', 0))
        y = float(attrs.get('y', 0))

        transform.append(
            "translate(%s,%s)" % (x, y)
        )

    if 'scaleX' in attrs or 'scaleY' in attrs:
        transform.append("scale(%s,%s)" % (
            float(attrs.get('scaleX', '1')), 
            float(attrs.get('scaleY', '1')),
        ))

    if 'rotation' in attrs:
        transform.append("rotate(%s)" % attrs['rotation'])

    return {
        'transform': ' '.join(transform)
    } if len(transform) > 0 else {}


def get_attribute(element, attrname):
    if element is None:
        return None
    if attrname in element.attrib:
        return element.attrib[attrname]
    return get_attribute(element.getparent(), attrname)


def populate_transform(element):
    for ch in element:
        if 'transform' in ch.attrib:
            # ch.set('transform', '%s %s' % (ch.attrib['transform'], element.attrib['transform']))
            ch.set('transform', '%s %s' % (element.attrib['transform'], ch.attrib['transform']))
        else:
            ch.set('transform', element.attrib['transform'])
        if ch.tag in ('clipPath', 'g'):
            populate_transform(ch)
    del element.attrib['transform']



from lxml import etree
from PIL import Image, ImageFont, ImageDraw
import logging

from ..utils import process_transform, get_attribute
from ..utils.fonts import get_font
from .default import DefaultTagProcessor


logger = logging.getLogger(__name__)


def get_recursive_data(element):
    text = element.text or ''
    for e in element:
        text += get_recursive_data(e)
    return text


def process_font(**attrs):
    params = {}
    if 'fontFamily' in attrs:
        params['font-family'] = attrs['fontFamily']
    if 'fontSize' in attrs:
        params['font-size'] = attrs['fontSize']
    if 'fontWeight' in attrs:
        params['font-weight'] = attrs['fontWeight']
    if 'color' in attrs:
        params['fill'] = attrs['color']
    if 'textAlign' in attrs:
        params['text-anchor'] = {
            'left': 'start',
            'center': 'middle',
            'right': 'end',
        }[attrs['textAlign']]
    if 'lineHeight' in attrs:
        params['line-height'] = attrs['lineHeight']
    if 'fontStyle' in attrs:
        params['font-style'] = attrs['fontStyle']
    return params


processors = {}
logger = logging.getLogger(__name__)


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('RichText')
class RichTextTagProcessor(DefaultTagProcessor):

    def on_start(self, target):
        target._p_count = -1
        params = {}
        params.update(process_transform(**target.attrs))
        params.update(process_font(**target.attrs))
        target.element = etree.SubElement(
            target.element,
            "text",
            **params
        )

    def on_end(self, target):
        family = target.element.get('font-family')
        size = target.element.get('font-size')
        text = get_recursive_data(target.element)
        if size is not None:
            size = float(size)
            font_object = get_font(family, fontdir=target.fontdir, size=round(size))
        else:
            font_object = get_font(family, fontdir=target.fontdir)
            fw, fh = font_object.getsize(text)
            size = fh
        asc, desc = font_object.getmetrics()
        target.element.attrib['y'] = str(size - desc)
        target.element = target.element.getparent()
        logger.debug("Y offset for text: '%s' is %s" % (text, size - desc))


@tag_processor('p')
class PTagProcessor(DefaultTagProcessor):

    def on_start(self, target):
        target._p_count += 1
        params = {}
        params.update(process_font(**target.attrs))
        params['x'] = '0'
        dy = max([
            float(get_attribute(target.element, 'line-height') or '0'),
            float(get_attribute(target.element, 'font-size') or '0'),
        ])
        if target._p_count > 0:
            params['dy'] = str(dy)
        target.element = etree.SubElement(
            target.element,
            "tspan",
            **params
        )


@tag_processor('span')
class SpanTagProcessor(PTagProcessor):
    def on_start(self, target):
        params = {}
        params.update(process_font(**target.attrs))
        target.element = etree.SubElement(
            target.element,
            "tspan",
            **params
        )

    def on_data(self, target):
        super(SpanTagProcessor, self).on_data(target)
        typographic_case = target.get_attribute('typographicCase')
        if typographic_case:
            if typographic_case == "uppercase":
                target.element.text = target.text.upper()
            if typographic_case == "lowercase":
                target.element.text = target.text.lower()



from lxml import etree

from ..utils import process_transform
from .default import DefaultTagProcessor


processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('Path')
class PathTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        params['d'] = target.attrs['data']
        params.update(process_transform(**target.attrs))
        target.element = etree.SubElement(
            target.element,
            "path",
            **params
        )


@tag_processor('Rect')
class RectTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        if 'radiusX' in target.attrs:
            params['rx'] = target.attrs['radiusX']
        if 'radiusY' in target.attrs:
            params['ry'] = target.attrs['radiusY']
        if 'alpha' in target.attrs:
            params['opacity'] = target.attrs['alpha']
        params['fill-opacity'] = '0'
        target.element = etree.SubElement(
            target.element,
            "rect",
            x="%s" % target.attrs.get("x", "0"),
            y="%s" % target.attrs.get("y", "0"),
            width="%s" % target.attrs.get("width"),
            height="%s" % target.attrs.get("height"),
            **params
        )


@tag_processor('Ellipse')
class EllipseTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        if 'alpha' in target.attrs:
            params['opacity'] = target.attrs['alpha']
        target.element = etree.SubElement(
            target.element,
            "ellipse",
            cx="%s" % (float(target.attrs.get("x")) + float(target.attrs.get("width")) / 2),
            cy="%s" % (float(target.attrs.get("y")) + float(target.attrs.get("height")) / 2),
            rx="%s" % (float(target.attrs.get("width")) / 2),
            ry="%s" % (float(target.attrs.get("height")) / 2),
            **params
        )



@tag_processor('Line')
class LineTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        params = {}
        target.element = etree.SubElement(
            target.element,
            "line",
            x1="%s" % target.attrs.get("xFrom", '0'),
            y1="%s" % target.attrs.get("yFrom", '0'),
            x2="%s" % target.attrs.get("xTo", '0'),
            y2="%s" % target.attrs.get("yTo", '0'),
            **params
        )



from setuptools import setup, find_packages


EXCLUDE_FROM_PACKAGES = []

setup(
    name='fxg2svg',
    version='0.1.9',
    description='FXG to SVG converter',
    url='https://bitbucket.org/suhain/fxg2svg',
    author='suhain',
    author_email='suhain93@gmail.com',
    license='MIT',
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    install_requires=[
        'Pillow==3.2.0',
        'lxml==3.6.0',
    ],
    entry_points={
        'console_scripts': [
            'fxg2svg = fxg2svg.convert:main', 
        ]
    },
    classifiers=(
        b'Programming Language :: Python :: 3.4',
        b'Environment :: Console',
        b'Development Status :: 3 - Alpha',
        b'Intended Audience :: Developers',
        b'License :: OSI Approved :: MIT License',
    )
)



import logging
import os
from PIL import ImageFont
import re


logger = logging.getLogger(__name__)
insensitive_re = re.compile(re.escape(' Regular'), re.IGNORECASE)


class FontNotFoundError(Exception):
    pass


def get_filepaths(directory):
    file_paths = []
    for root, directories, files in os.walk(directory):
        for filename in files:
            filepath = os.path.join(root, filename)
            file_paths.append(filepath)  
    return file_paths


def get_font_alias(font):
    return insensitive_re.sub("", " ".join(font.getname()))


def get_font_file(name, f, fontdir):
    for filepath in get_filepaths(fontdir):
        try:
            font = ImageFont.truetype(filepath)
        except OSError:
            logger.debug('%s is not a font' % filepath)
        else:
            if get_font_alias(font) == name:
                return filepath
        finally:
            del font


def get_font(name, f='ttf', fontdir=None, size=None):
    if fontdir is None:
        raise FontNotFoundError("Font '%s' not found." % name)
    fontpath = get_font_file(name, f, fontdir)
    if fontpath is None:
        raise FontNotFoundError("Font '%s' not found." % name)
    if size is None:
        font = ImageFont.truetype(fontpath)
    else:
        font = ImageFont.truetype(fontpath, size=size)
    return font



"""
FlashXmlGraphics to ScalableVectorGraphic converter.
"""

from lxml import etree

from .svgtarget import SVGTarget


def fxg2svg(fxg_bytes, fontdir=None):
    return etree.parse(fxg_bytes, etree.XMLParser(target=SVGTarget(fontdir=fontdir)))
